package org.acme.websockets;

import java.net.URI;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.TimeUnit;

import javax.inject.Inject;
import javax.websocket.ClientEndpoint;
import javax.websocket.ContainerProvider;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import io.quarkus.test.common.http.TestHTTPResource;
import io.quarkus.test.junit.QuarkusTest;

@QuarkusTest
public class ChatTest {

	private static final LinkedBlockingDeque<String> MESSAGES = new LinkedBlockingDeque<>();

	@TestHTTPResource("/chat/stu")
	URI uri;

	@Inject
	Bean bean;

	@Test
	public void testWebsocketChat() throws Exception {
		Client client = new Client();
		client.setBean(bean);
		try (Session session = ContainerProvider.getWebSocketContainer().connectToServer(client, uri)) {
			Assertions.assertEquals("CONNECT", MESSAGES.poll(10, TimeUnit.SECONDS));
			Assertions.assertEquals("User stu joined", MESSAGES.poll(10, TimeUnit.SECONDS));
			session.getAsyncRemote().sendText("hello world");
			Assertions.assertEquals(">> stu: hello world", MESSAGES.poll(10, TimeUnit.SECONDS));
		}
	}

	@ClientEndpoint
	public static class Client {

		private Bean bean;

		void setBean(Bean bean) {
			this.bean = bean;
		}

		@OnOpen
		public void open(Session session) {
			MESSAGES.add("CONNECT");
			// Send a message to indicate that we are ready,
			// as the message handler may not be registered immediately after this callback.
//            session.getAsyncRemote().sendText("_ready_");
		}

		@OnMessage
		void message(String msg) {
			bean.doSomething();
			MESSAGES.add(msg);
		}

	}

}